USE [master]
GO

/* TestMetadata */

IF  NOT EXISTS (SELECT name FROM sys.databases WHERE name = N'TestMetadata')
CREATE DATABASE [TestMetadata] ON  PRIMARY ( 
	NAME = N'TestMetadata', 
	FILENAME = N'D:\SQLData\TestMetadata.mdf', 
	SIZE = 3072KB, 
	MAXSIZE = UNLIMITED, 
	FILEGROWTH = 1024KB 
)
LOG ON ( 
	NAME = N'TestMetadata_log', 
	FILENAME = N'D:\SQLData\TestMetadata_log.ldf', 
	SIZE = 1024KB, 
	MAXSIZE = 2048GB, 
	FILEGROWTH = 10%
)
GO

ALTER DATABASE [TestMetadata] SET COMPATIBILITY_LEVEL = 100
GO

use TestMetadata
go

/* Set up the test scenario */

IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[NumberOfRows]') AND type in (N'U'))
begin
	create table NumberOfRows (
		iteration tinyint,
		numberOfRows int
	);

	insert into NumberOfRows values (1, 100000);
	insert into NumberOfRows values (2, 1000000);
	insert into NumberOfRows values (3, 10000000);
end

IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[NumberOfAttributes]') AND type in (N'U'))
begin
	create table NumberOfAttributes (
		iteration tinyint,
		numberOfAttributes int
	);

	insert into NumberOfAttributes values (1, 6);
	insert into NumberOfAttributes values (2, 12);
	insert into NumberOfAttributes values (3, 24);
END;

IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[QueriedAttributes]') AND type in (N'U'))
begin
	create table QueriedAttributes (
		iteration tinyint,
		numberOfAttributes int,
		queryType varchar(255),
		queriedAttributes int
	);

	insert into QueriedAttributes values (1, 6, 'Single', 1);
	insert into QueriedAttributes values (2, 6, '1/3', 2);
	insert into QueriedAttributes values (3, 6, '2/3', 4);
	insert into QueriedAttributes values (4, 6, 'All', 6);
	insert into QueriedAttributes values (1, 12, 'Single', 1);
	insert into QueriedAttributes values (2, 12, '1/3', 4);
	insert into QueriedAttributes values (3, 12, '2/3', 8);
	insert into QueriedAttributes values (4, 12, 'All', 12);
	insert into QueriedAttributes values (1, 24, 'Single', 1);
	insert into QueriedAttributes values (2, 24, '1/3', 8);
	insert into QueriedAttributes values (3, 24, '2/3', 16);
	insert into QueriedAttributes values (4, 24, 'All', 24);
END;

IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[QueryTimes]') AND type in (N'U'))
create table QueryTimes (
	id int identity(1,1) not null primary key, 
	suite char(1),
	data char(3),
	modelType char(3),
	modeledAttributes int,
	queryType varchar(255),
	queryIteration int,
	queriedAttributes int,
	numberOfRows int,
	startingTime datetime, 
	endingTime datetime
);


IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Queries]') AND type in (N'U'))
create table Queries (
	id int identity(1,1) not null primary key, 
	data char(3),
	modelType char(3),
	modeledAttributes int,
	numberOfRows int,
	query varchar(max)
);


IF  NOT EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[DatabaseSizes]') AND type in (N'U'))
create table DatabaseSizes (
	id int identity(1,1) not null primary key, 
	data char(3),
	modelType char(3),
	modeledAttributes int,
	numberOfRows int,
	size bigint
);

go

-- this view calculates the ratios

IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[QueryRatios]'))
DROP VIEW [dbo].[QueryRatios]
GO

create view QueryRatios as
with durations as (
	select 
		id,
		suite,
		data,
		modelType,
		modeledAttributes,
		queryType,
		queryIteration,
		queriedAttributes, 
		numberOfRows,
		DATEDIFF(ms, startingTime, endingTime) as duration
	from 
		TestMetadata.dbo.QueryTimes	
)
select
	suite,
	data,
	modeledAttributes,
	numberOfRows,
	queryType,
	queriedAttributes,
	min(case when modelType = '3NF' then cast(durationMin as float) else null end) as durationMin3NF,
	min(case when modelType = 'AM' then cast(durationMin as float) else null end) as durationMinAM,
	min(case when modelType = '3NF' then cast(durationMax as float) else null end) as durationMax3NF,
	min(case when modelType = 'AM' then cast(durationMax as float) else null end) as durationMaxAM,
	min(case when modelType = '3NF' then cast(durationAvg as float) else null end) as durationAvg3NF,
	min(case when modelType = 'AM' then cast(durationAvg as float) else null end) as durationAvgAM,
	cast(case 
		when min(case when modelType = '3NF' then cast(durationMin as float) else null end) > min(case when modelType = 'AM' then cast(durationMin as float) else null end)
		then (+1) * min(case when modelType = '3NF' then cast(durationMin as float) else null end) / min(case when modelType = 'AM' then cast(durationMin as float) else null end)
		when min(case when modelType = 'AM' then cast(durationMin as float) else null end) > min(case when modelType = '3NF' then cast(durationMin as float) else null end)
		then (-1) * min(case when modelType = 'AM' then cast(durationMin as float) else null end) / min(case when modelType = '3NF' then cast(durationMin as float) else null end)
		else 1
	end as decimal(4,1)) as ratioMin,
	cast(case 
		when min(case when modelType = '3NF' then cast(durationMax as float) else null end) > min(case when modelType = 'AM' then cast(durationMax as float) else null end)
		then (+1) * min(case when modelType = '3NF' then cast(durationMax as float) else null end) / min(case when modelType = 'AM' then cast(durationMax as float) else null end)
		when min(case when modelType = 'AM' then cast(durationMax as float) else null end) > min(case when modelType = '3NF' then cast(durationMax as float) else null end)
		then (-1) * min(case when modelType = 'AM' then cast(durationMax as float) else null end) / min(case when modelType = '3NF' then cast(durationMax as float) else null end)
		else 1
	end as decimal(4,1)) as ratioMax,
	cast(case 
		when min(case when modelType = '3NF' then cast(durationAvg as float) else null end) > min(case when modelType = 'AM' then cast(durationAvg as float) else null end)
		then (+1) * min(case when modelType = '3NF' then cast(durationAvg as float) else null end) / min(case when modelType = 'AM' then cast(durationAvg as float) else null end)
		when min(case when modelType = 'AM' then cast(durationAvg as float) else null end) > min(case when modelType = '3NF' then cast(durationAvg as float) else null end)
		then (-1) * min(case when modelType = 'AM' then cast(durationAvg as float) else null end) / min(case when modelType = '3NF' then cast(durationAvg as float) else null end)
		else 1
	end as decimal(4,1)) as ratioAvg	
from (
	select
		d.suite,
		d.data,
		d.modelType,
		d.modeledAttributes,
		d.queryType,
		d.queriedAttributes, 
		d.numberOfRows,	
		MIN(d.duration) as durationMin,
		MAX(d.duration) as durationMax,
		AVG(d.duration) as durationAvg
	from
		durations d
	group by
		d.suite,
		d.data,
		d.modelType,
		d.modeledAttributes,
		d.queryType,
		d.queriedAttributes, 
		d.numberOfRows		
	) x
group by
	suite,
	data,
	modeledAttributes,
	queryType,
	queriedAttributes,
	numberOfRows

/*
select 
	suite, 
	data, 
	modeledAttributes,
	numberOfRows,
	queryType, 
	queriedAttributes, 
	ratioMax,
	round(ratioMax, 0),
	case 
		when round(ratioMax, 0) = 1 then cast(cast(round(ratioMax, 0) as int) as varchar(10))
		when round(ratioMax, 0) = -1 then cast(cast(round((-1) * ratioMax, 0) as int) as varchar(10))
		when round(ratioMax, 0) > 1 then '+' + cast(cast(round(ratioMax, 0) as int) as varchar(10))
		else cast(cast(round(ratioMax, 0) as int) as varchar(10))
	end
from
	TestMetadata.dbo.QueryRatios
order by	
	suite,
	data,
	modeledAttributes,
	queriedAttributes,
	numberOfRows

select 
	*, 
	DATEDIFF(ms, startingTime, endingTime) as duration
from 
	TestMetadata.dbo.QueryTimes	
order by
	startingTime desc

select 
	*,
	cast('<![CDATA[' + query + ']]>' as xml)
from 
	TestMetadata.dbo.Queries

select * from TestMetadata.dbo.DatabaseSizes	

-- truncate table TestMetadata.dbo.Queries

 delete from TestMetadata.dbo.Queries where data = 'UDS'
 delete from TestMetadata.dbo.QueryTimes where data = 'UDS'
 delete from TestMetadata.dbo.DatabaseSizes where data = 'UDS'

 delete from TestMetadata.dbo.Queries where data = 'HDS'
 delete from TestMetadata.dbo.QueryTimes where data = 'HDS' and suite = 'B'
 delete from TestMetadata.dbo.DatabaseSizes where data = 'HDS'

 delete from TestMetadata.dbo.Queries where data = 'HSS'
 delete from TestMetadata.dbo.QueryTimes where data = 'HSS' 
 delete from TestMetadata.dbo.DatabaseSizes where data = 'HSS'

 delete from TestMetadata.dbo.Queries where data = 'HSH'
 delete from TestMetadata.dbo.QueryTimes where data = 'HSH'
 delete from TestMetadata.dbo.DatabaseSizes where data = 'HSH'
 

*/
	