use master;
go

-- time to run: 10 minutes

-- test data that is copied to the test databases later

IF EXISTS (SELECT name FROM sys.databases WHERE name = N'TestData')
DROP DATABASE [TestData];

CREATE DATABASE [TestData] ON  PRIMARY ( 
	NAME = N'TestData', 
	FILENAME = N'D:\SQLData\TestData.mdf', 
	SIZE = 10240000KB, 
	MAXSIZE = UNLIMITED, 
	FILEGROWTH = 10240000KB
)
LOG ON ( 
	NAME = N'TestData_log', 
	FILENAME = N'D:\SQLData\TestData_log.ldf', 
	SIZE = 5120000KB, 
	MAXSIZE = 2048GB, 
	FILEGROWTH = 5120000KB
)
GO

ALTER DATABASE [TestData] SET COMPATIBILITY_LEVEL = 100
GO

use TestData
go

-- approx. 128MB data used to dirty down the disk caches

create table DirtyCache (
	a char(32),
	b char(32),
	c char(32),
	d char(32)
);

WITH dirt(num) AS (
SELECT
	1
UNION ALL
SELECT
	num + 1
FROM
	dirt
WHERE
	num < 1000000
)
insert into DirtyCache
select
	'0123456789abcdef0123456789ABCDEF',	
	'0123456789abcdef0123456789ABCDEF',	
	'0123456789abcdef0123456789ABCDEF',	
	'0123456789abcdef0123456789ABCDEF'	
from
	dirt
option (maxrecursion 0);


create table ANATT_AnchorAttribute_UDS (
	AN_ID int identity(1,1) not null, 
	ANATT_AnchorAttribute tinyint not null		
);

create table ANATT_AnchorAttribute_HDS_0 (
	AN_ID int identity(1,1) not null, 
	ANATT_AnchorAttribute money not null		
);

create table ANATT_AnchorAttribute_HDH_0 (
	AN_ID int not null, 
	ANATT_AnchorAttribute money not null, 
	ANATT_ValidFrom date not null		
);

create table KNO_Knot_1 (
	KNO_ID bit not null,
	KNO_Value char(4) not null
);
insert into KNO_Knot_1
values (0, 'Even');
insert into KNO_Knot_1
values (1, 'Odd');

create table ANATT_AnchorAttribute_HDS_1 (
	AN_ID int identity(1,1) not null, 
	KNO_ID bit not null		
);

create table ANATT_AnchorAttribute_HDS_2 (
	AN_ID int identity(1,1) not null, 
	ANATT_AnchorAttribute varchar(10) not null		
);

create table KNO_Knot_3 (
	KNO_ID tinyint not null,
	KNO_Value varchar(8) not null
)
insert into KNO_Knot_3
values(0, 'Alpha');
insert into KNO_Knot_3
values(1, 'November');
insert into KNO_Knot_3
values(2, 'Charlie');
insert into KNO_Knot_3
values(3, 'Hotel');
insert into KNO_Knot_3
values(4, 'Oscar');
insert into KNO_Knot_3
values(5, 'Romeo');

create table ANATT_AnchorAttribute_HDS_3 (
	AN_ID int identity(1,1) not null, 
	KNO_ID tinyint not null		
);

create table ANATT_AnchorAttribute_HDS_4 (
	AN_ID int identity(1,1) not null, 
	ANATT_AnchorAttribute datetime not null
);

create table ANATT_AnchorAttribute_HDS_5 (
	AN_ID int identity(1,1) not null, 
	ANATT_AnchorAttribute char(32) not null
);


declare @data table (
	id int identity(1,1) not null primary key,
	value_UDS tinyint,			-- 1 byte
	value_HDS_0 money,			-- 8 bytes
	value_HDS_1 bit,			-- 1 bit stored as 1 byte referring 4 bytes
	value_HDS_2 varchar(10),	-- between 3 and 12 bytes (number of chars + 2)
	value_HDS_3 tinyint,		-- 1 byte referring max 10 bytes
	value_HDS_4 datetime,		-- 8 bytes
	value_HDS_5 char(32)		-- 32 bytes
);

declare @curIter int;
declare @maxIter int;
declare @maxRows int;
declare @subRows int;

set @maxIter = 100;

select
	@maxRows = MAX(numberOfRows)
from
	TestMetadata.dbo.NumberOfRows;
	
set @curIter = 0;

select @subRows = @maxRows / @maxIter;
	
WITH row(num) AS (
SELECT
	1
UNION ALL
SELECT
	num + 1
FROM
	row
WHERE
	num < @subRows
)
insert into @data (
	value_UDS,	
	value_HDS_0,
	value_HDS_1,
	value_HDS_2,
	value_HDS_3,
	value_HDS_4,
	value_HDS_5	
)
select
	cast(num % 10 as tinyint) as value_UDS,	
	cast(1000000 * (rand(num) - rand(@subRows - num)) as money) as value_HDS_0,
	cast(num % 2 as bit) as value_HDS_1,
	cast(num as varchar(10)) as value_HDS_2, 
	cast(num % 6 as tinyint) as value_HDS_3, 
	cast('2010-01-01' as datetime) + 365 * rand(num) as value_HDS_4, 
	convert(char(32), hashbytes('md5', cast(num as varchar(10))), 2) as value_HDS_5	
from
	row
option (maxrecursion 0);

while @curIter < @maxIter
begin
	insert into ANATT_AnchorAttribute_UDS (
		ANATT_AnchorAttribute
	)
	select value_UDS from @data order by id asc;		

	insert into ANATT_AnchorAttribute_HDS_0 (
		ANATT_AnchorAttribute
	)
	select value_HDS_0 from @data order by id asc;		

	insert into ANATT_AnchorAttribute_HDS_1 (
		KNO_ID
	)
	select value_HDS_1 from @data order by id asc;

	insert into ANATT_AnchorAttribute_HDS_2 (
		ANATT_AnchorAttribute
	)
	select value_HDS_2 from @data order by id asc;

	insert into ANATT_AnchorAttribute_HDS_3 (
		KNO_ID
	)
	select value_HDS_3 from @data order by id asc;

	insert into ANATT_AnchorAttribute_HDS_4 (
		ANATT_AnchorAttribute
	)
	select value_HDS_4 from @data order by id asc;

	insert into ANATT_AnchorAttribute_HDS_5 (
		ANATT_AnchorAttribute
	)
	select value_HDS_5 from @data order by id asc;
	
	set @curIter = @curIter + 1;
end

-- first version
insert into ANATT_AnchorAttribute_HDH_0 
select
	anatt.AN_ID,
	anatt.ANATT_AnchorAttribute,
	'1972-02-13'
from
	ANATT_AnchorAttribute_HDS_0 anatt

-- second version	
insert into ANATT_AnchorAttribute_HDH_0 
select
	anatt.AN_ID,
	(-1) * anatt.ANATT_AnchorAttribute,
	'2009-09-21'
from
	ANATT_AnchorAttribute_HDS_0 anatt
	
	
/*
HDS data
-----------
0 - 201 MB
1 - 135 MB
2 - 211 MB
3 - 135 MB
4 - 203 MB
5 - 436 MB

total - 1321 MB per 6 tables

tables approx. db size
------------------------
6  = 1  * 1321 = 1321 MB
12 = 2  * 1321 = 2642 MB
24 = 4  * 1321 = 5284 MB

*/
	
/*
select top 10 * from ANATT_AnchorAttribute_UDS
select top 10 * from ANATT_AnchorAttribute_HDS_0
select top 10 * from ANATT_AnchorAttribute_HDS_1
select top 10 * from ANATT_AnchorAttribute_HDS_2
select top 10 * from ANATT_AnchorAttribute_HDS_3
select top 10 * from ANATT_AnchorAttribute_HDS_4
select top 10 * from ANATT_AnchorAttribute_HDS_5

*/

