USE [master]
GO

-- uniform dense static

IF  EXISTS (SELECT name FROM sys.databases WHERE name = N'Test3NF_UDS')
DROP DATABASE [Test3NF_UDS];
create database [Test3NF_UDS];

IF  EXISTS (SELECT name FROM sys.databases WHERE name = N'TestAM_UDS')
DROP DATABASE [TestAM_UDS];
create database [TestAM_UDS];

GO

-- configuration parameters

declare @query_repeats int;
set @query_repeats = 2;

declare @suiteA bit;
set @suiteA = 1;

declare @suiteB bit;
set @suiteB = 1;

-- iteration counters

declare @rows_iter int;
declare @size_iter int;
declare @qatt_iter int;

set @rows_iter = 1;
set @size_iter = 1;
set @qatt_iter = 1;

declare @rows_max int;
declare @size_max int;
declare @qatt_max int;

select @rows_max = MAX(iteration) from TestMetadata.dbo.NumberOfRows;
select @size_max = MAX(iteration) from TestMetadata.dbo.NumberOfAttributes;
select @qatt_max = MAX(iteration) from TestMetadata.dbo.QueriedAttributes;


-- loop through the measure points

while @size_iter <= @size_max
begin
	declare @numberOfAttributes int;
	select 
		@numberOfAttributes = numberOfAttributes 
	from 
		TestMetadata.dbo.NumberOfAttributes 
	where 
		iteration = @size_iter;
	
	set @rows_iter = 1;
	while @rows_iter <= @rows_max
	begin	
		declare @numberOfRows int;
		select 
			@numberOfRows = numberOfRows 
		from 
			TestMetadata.dbo.NumberOfRows 
		where 
			iteration = @rows_iter;

		-- the query will be built into this variable
		declare @sql varchar(max);

		-- first build the anchor model query
		select @sql = '
		use master;
		
		IF  EXISTS (SELECT name FROM sys.databases WHERE name = N''TestAM_UDS'')
		DROP DATABASE [TestAM_UDS]

		CREATE DATABASE [TestAM_UDS] ON  PRIMARY ( 
			NAME = N''TestAM_UDS'', 
			FILENAME = N''D:\SQLData\TestAM_UDS.mdf'', 
			SIZE = 10GB, 
			MAXSIZE = UNLIMITED, 
			FILEGROWTH = 10GB 
		)
		LOG ON ( 
			NAME = N''TestAM_UDS_log'', 
			FILENAME = N''D:\SQLData\TestAM_UDS_log.ldf'', 
			SIZE = 5GB, 
			MAXSIZE = 2048GB, 
			FILEGROWTH = 5GB 
		)

		ALTER DATABASE [TestAM_UDS] SET COMPATIBILITY_LEVEL = 100
		ALTER DATABASE [TestAM_UDS] SET AUTO_CREATE_STATISTICS ON 
		ALTER DATABASE [TestAM_UDS] SET AUTO_UPDATE_STATISTICS ON 
		ALTER DATABASE [TestAM_UDS] SET RECOVERY SIMPLE 
		
		USE TestAM_UDS;

		CREATE TABLE [AN_Anchor] ( 
			AN_ID int not null
		);		
		insert into AN_Anchor
		select top ' + cast(@numberOfRows as varchar(10)) + ' AN_ID from TestData.dbo.ANATT_AnchorAttribute_UDS;
		alter table AN_Anchor add constraint pkAN_Anchor primary key clustered (AN_ID asc);
		update statistics AN_Anchor;
		'

		declare @attr_iter int;
		set @attr_iter = 1;
		
		while @attr_iter <= @numberOfAttributes
		begin
			select @sql = @sql + '
			CREATE TABLE [ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + '] ( 
				AN_ID int not null,
				ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' tinyint not null
			);		
			insert into ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + '
			select top ' + cast(@numberOfRows as varchar(10)) + ' * from TestData.dbo.ANATT_AnchorAttribute_UDS
			alter table ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' add constraint pkANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' primary key clustered (AN_ID asc);
			alter table ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' add constraint fkANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' foreign key (AN_ID) references AN_Anchor (AN_ID);
			update statistics ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ';
			'	
			
			set @attr_iter = @attr_iter + 1;
		end

		select @sql = @sql + '
		exec TestAM_UDS.dbo.sp_executesql N''
		create view lAN_Anchor as 
		SELECT 
			[AN].AN_ID				
		'

		set @attr_iter = 1;

		while @attr_iter <= @numberOfAttributes
		begin
			select @sql = 
			@sql + ',[' + cast(@attr_iter as varchar(10)) + '].ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + '
			'
			
			set @attr_iter = @attr_iter + 1;
		end

		select @sql = @sql + '
		from 
			AN_Anchor [AN]
		'

		set @attr_iter = 1;

		while @attr_iter <= @numberOfAttributes
		begin
			select @sql = 
			@sql + 'left join ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' [' + cast(@attr_iter as varchar(10)) + ']
			on [' + cast(@attr_iter as varchar(10)) + '].AN_ID = [AN].AN_ID
			'
			
			set @attr_iter = @attr_iter + 1;
		end
		
		select @sql = @sql + '''
		
		declare @iter int;
		declare @runs int;
		declare @identity int;		
		'

		declare @query varchar(max);
		declare @queriedAttributes int;
		declare @queryType varchar(255);
		declare @qsub_iter int;


		-- without where conditions
		if @suiteA = 1
		begin
			set @qatt_iter = 1;		
			while @qatt_iter <= @qatt_max
			begin
				select 
					@queriedAttributes = queriedAttributes,
					@queryType = queryType
				from 
					TestMetadata.dbo.QueriedAttributes 
				where 
					iteration = @qatt_iter 
				and 
					numberOfAttributes = @numberOfAttributes;					
				
				select @query = '
				select
					0 as dummy
				'
				
				set @qsub_iter = 1;

				while @qsub_iter <= @queriedAttributes
				begin
					select @query = @query 
					+ ', count(case when ANATT_AnchorAttribute_' 
					+ cast(@qsub_iter as varchar(10)) 
					+ ' = 0 then 1 else null end) as value_' 
					+ cast(@qsub_iter as varchar(10)) 
					+ '
					'
					
					set @qsub_iter = @qsub_iter + 1;
				end

				select @query = @query + '
				into
					#UDS_AM_A'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ '
				from 
					lAN_Anchor;			
				'
				
				select @sql = @sql + '
				
				set @runs = ' + cast(@query_repeats as varchar(10)) + '
				set @iter = 0;

				while @iter < @runs
				begin
					if object_id(''tempdb..#UDS_AM_A'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ ''') is not null
					drop table #UDS_AM_A'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ ';
				
					-- clear cache
					CHECKPOINT;
					DBCC DROPCLEANBUFFERS WITH NO_INFOMSGS;
					DBCC FREEPROCCACHE WITH NO_INFOMSGS;
					DBCC FREESESSIONCACHE WITH NO_INFOMSGS;
					DBCC FREESYSTEMCACHE (''All'') WITH NO_INFOMSGS;

					-- make disk cache dirty
					select top 0 
						*
					into 
						DirtyCache
					from
						TestData.dbo.DirtyCache;
					
					insert into DirtyCache	
					select
						*
					from 
						TestData.dbo.DirtyCache
						
					drop table DirtyCache;

					insert into TestMetadata.dbo.QueryTimes(
						suite,
						data,
						modelType,
						modeledAttributes,
						queryType,
						queryIteration,
						queriedAttributes,
						numberOfRows,
						startingTime
					)
					values(
						''A'',
						''UDS'',
						''AM'',
						' + cast(@numberOfAttributes as varchar(10)) + ',
						''' + @queryType + ''',
						@iter + 1,
						' + cast(@queriedAttributes as varchar(10)) + ',
						' + cast(@numberOfRows as varchar(10)) + ',
						GETDATE()
					);
					
					' + @query + '

					update TestMetadata.dbo.QueryTimes 
					set
						endingTime = GETDATE()
					where
						endingTime is null
					and
						suite = ''A''
					and
						data = ''UDS''
					and
						modelType = ''AM''
					and
						modeledAttributes = ' + cast(@numberOfAttributes as varchar(10)) + '
					and
						queryIteration = @iter + 1
					and
						queriedAttributes = ' + cast(@queriedAttributes as varchar(10)) + '
					and
						numberOfRows = ' + cast(@numberOfRows as varchar(10)) + '
						
					set @iter = @iter + 1;
				end			
				'
				
				set @qatt_iter = @qatt_iter + 1;
			end
		end

		-- with where conditions
		if @suiteB = 1
		begin		
			set @qatt_iter = 1;		
			while @qatt_iter <= @qatt_max
			begin
				select 
					@queriedAttributes = queriedAttributes,
					@queryType = queryType
				from 
					TestMetadata.dbo.QueriedAttributes 
				where 
					iteration = @qatt_iter 
				and 
					numberOfAttributes = @numberOfAttributes;					
				
				select @query = '
				select
					count(*) as numberOfRecords
				'
				
				select @query = @query + '
				into
					#UDS_AM_B'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ '
				from 
					lAN_Anchor
				where	
					1 = 1		
				'
				
				set @qsub_iter = 1;

				while @qsub_iter <= @queriedAttributes
				begin
					select @query = @query 
					+ 'and ANATT_AnchorAttribute_' 
					+ cast(@qsub_iter as varchar(10)) 
					+ ' = 0  
					'
									
					set @qsub_iter = @qsub_iter + 1;
				end
				
				select @sql = @sql + '
				
				set @runs = ' + cast(@query_repeats as varchar(10)) + '
				set @iter = 0;

				while @iter < @runs
				begin
					if object_id(''tempdb..#UDS_AM_B'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ ''') is not null
					drop table #UDS_AM_B'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ ';
				
					-- clear cache
					CHECKPOINT;
					DBCC DROPCLEANBUFFERS WITH NO_INFOMSGS;
					DBCC FREEPROCCACHE WITH NO_INFOMSGS;
					DBCC FREESESSIONCACHE WITH NO_INFOMSGS;
					DBCC FREESYSTEMCACHE (''All'') WITH NO_INFOMSGS;
					
					-- make disk cache dirty
					select top 0 
						*
					into 
						DirtyCache
					from
						TestData.dbo.DirtyCache;
					
					insert into DirtyCache	
					select
						*
					from 
						TestData.dbo.DirtyCache;
						
					drop table DirtyCache;

					insert into TestMetadata.dbo.QueryTimes(
						suite,
						data,
						modelType,
						modeledAttributes,
						queryType,
						queryIteration,
						queriedAttributes,
						numberOfRows,
						startingTime
					)
					values(
						''B'',
						''UDS'',
						''AM'',
						' + cast(@numberOfAttributes as varchar(10)) + ',
						''' + @queryType + ''',
						@iter + 1,
						' + cast(@queriedAttributes as varchar(10)) + ',
						' + cast(@numberOfRows as varchar(10)) + ',
						GETDATE()
					);
					
					' + @query + '

					update TestMetadata.dbo.QueryTimes 
					set
						endingTime = GETDATE()
					where
						endingTime is null
					and
						suite = ''B''
					and
						data = ''UDS''
					and
						modelType = ''AM''
					and
						modeledAttributes = ' + cast(@numberOfAttributes as varchar(10)) + '
					and
						queryIteration = @iter + 1
					and
						queriedAttributes = ' + cast(@queriedAttributes as varchar(10)) + '
					and
						numberOfRows = ' + cast(@numberOfRows as varchar(10)) + '
						
					set @iter = @iter + 1;
				end			
				'
				
				set @qatt_iter = @qatt_iter + 1;
			end
		end
		
		select @sql = @sql + '
		DBCC SHRINKFILE (1, 1) -- data
		DBCC SHRINKFILE (2, 1) -- log
		
		declare @size bigint;
		select 
			@size = SUM(size) 
		from 
			TestAM_UDS.sys.database_files 
		where 
			type_desc = ''ROWS'';

		insert into TestMetadata.dbo.DatabaseSizes (
			data,
			modelType,
			modeledAttributes,
			numberOfRows,
			size
		)
		values (
			''UDS'',
			''AM'',
			' + cast(@numberOfAttributes as varchar(10)) + ',
			' + cast(@numberOfRows as varchar(10)) + ',
			@size
		);			
		'
		
		insert into TestMetadata.dbo.Queries (
			data,
			modelType,
			modeledAttributes,
			numberOfRows,
			query	
		)
		values (
			'UDS',
			'AM',
			@numberOfAttributes,
			@numberOfRows,
			@sql
		);
		
		-- then build the 3NF query
		set @sql = '
		use master;
		
		IF  EXISTS (SELECT name FROM sys.databases WHERE name = N''Test3NF_UDS'')
		DROP DATABASE [Test3NF_UDS]

		CREATE DATABASE [Test3NF_UDS] ON  PRIMARY ( 
			NAME = N''Test3NF_UDS'', 
			FILENAME = N''D:\SQLData\Test3NF_UDS.mdf'', 
			SIZE = 10GB, 
			MAXSIZE = UNLIMITED, 
			FILEGROWTH = 10GB 
		)
		LOG ON ( 
			NAME = N''Test3NF_UDS_log'', 
			FILENAME = N''D:\SQLData\Test3NF_UDS_log.ldf'', 
			SIZE = 5GB, 
			MAXSIZE = 2048GB, 
			FILEGROWTH = 5GB 
		)

		ALTER DATABASE [Test3NF_UDS] SET COMPATIBILITY_LEVEL = 100
		ALTER DATABASE [Test3NF_UDS] SET AUTO_CREATE_STATISTICS ON 
		ALTER DATABASE [Test3NF_UDS] SET AUTO_UPDATE_STATISTICS ON 
		ALTER DATABASE [Test3NF_UDS] SET RECOVERY SIMPLE 
		
		use Test3NF_UDS;
		
		select top 0
			*
		into
			[3nfAN_Anchor]
		from
			TestAM_UDS.dbo.lAN_Anchor;
			
		declare @insert_iter int;
		declare @insert_max int;		
		declare @from_id int;
		declare @to_id int;
				
		set @insert_iter = 1;
		set @insert_max = 100;

		while @insert_iter <= @insert_max 
		begin
			select @from_id = (@insert_iter - 1) * (' + cast(@numberOfRows as varchar(10)) + ' / @insert_max) + 1;
			select @to_id = @insert_iter * (' + cast(@numberOfRows as varchar(10)) + ' / @insert_max);

			insert into [3nfAN_Anchor]
			select 
				*
			from
				TestAM_UDS.dbo.lAN_Anchor
			where
				AN_ID between @from_id and @to_id;
				
			set @insert_iter = @insert_iter + 1;
		end

		-- now we do not need the AM database any longer
		IF  EXISTS (SELECT name FROM sys.databases WHERE name = N''TestAM_UDS'')
		DROP DATABASE [TestAM_UDS];
		
		-- create an empty db so that the script validation will not break
		create database [TestAM_UDS];

		alter table [3nfAN_Anchor] add constraint pk3nfAN_Anchor primary key clustered (AN_ID asc);
		update statistics [3nfAN_Anchor];
					
		declare @iter int;
		declare @runs int;
		declare @identity int;		
		'

		set @query ='';
		
		-- without where condition
		if @suiteA = 1
		begin		
			set @qatt_iter = 1;		
			while @qatt_iter <= @qatt_max
			begin
				select 
					@queriedAttributes = queriedAttributes,
					@queryType = queryType 
				from 
					TestMetadata.dbo.QueriedAttributes 
				where 
					iteration = @qatt_iter 
				and 
					numberOfAttributes = @numberOfAttributes;					
				
				select @query = '
				select
					0 as dummy
				'
				
				set @qsub_iter = 1;

				while @qsub_iter <= @queriedAttributes
				begin
					select @query = @query 
					+ ', count(case when ANATT_AnchorAttribute_' 
					+ cast(@qsub_iter as varchar(10)) 
					+ ' = 0 then 1 else null end) as value_' 
					+ cast(@qsub_iter as varchar(10)) 
					+ '
					'
					
					set @qsub_iter = @qsub_iter + 1;
				end

				select @query = @query + '
				into
					#UDS_3NF_A'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ '
				from 
					[3nfAN_Anchor];			
				'
				
				select @sql = @sql + '
				
				set @runs = ' + cast(@query_repeats as varchar(10)) + ';
				set @iter = 0;

				while @iter < @runs
				begin
					if object_id(''tempdb..#UDS_3NF_A'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ ''') is not null
					drop table #UDS_3NF_A'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ ';

					-- clear cache
					CHECKPOINT;
					DBCC DROPCLEANBUFFERS WITH NO_INFOMSGS;
					DBCC FREEPROCCACHE WITH NO_INFOMSGS;
					DBCC FREESESSIONCACHE WITH NO_INFOMSGS;
					DBCC FREESYSTEMCACHE (''All'') WITH NO_INFOMSGS;
					
					-- make disk cache dirty
					select top 0 
						*
					into 
						DirtyCache
					from
						TestData.dbo.DirtyCache;
					
					insert into DirtyCache	
					select
						*
					from 
						TestData.dbo.DirtyCache
						
					drop table DirtyCache;

					insert into TestMetadata.dbo.QueryTimes(
						suite,
						data,
						modelType,
						modeledAttributes,
						queryType,
						queryIteration,
						queriedAttributes,
						numberOfRows,
						startingTime
					)
					values(
						''A'',
						''UDS'',
						''3NF'',
						' + cast(@numberOfAttributes as varchar(10)) + ',
						''' + @queryType + ''',
						@iter + 1,
						' + cast(@queriedAttributes as varchar(10)) + ',
						' + cast(@numberOfRows as varchar(10)) + ',
						GETDATE()
					);
					
					' + @query + '

					update TestMetadata.dbo.QueryTimes 
					set
						endingTime = GETDATE()
					where
						endingTime is null
					and
						suite = ''A''
					and
						data = ''UDS''
					and
						modelType = ''3NF''
					and
						modeledAttributes = ' + cast(@numberOfAttributes as varchar(10)) + '
					and
						queryIteration = @iter + 1
					and
						queriedAttributes = ' + cast(@queriedAttributes as varchar(10)) + '
					and
						numberOfRows = ' + cast(@numberOfRows as varchar(10)) + '
						
					set @iter = @iter + 1;
				end			
				'
				
				set @qatt_iter = @qatt_iter + 1;
			end
		end

		-- with where condition
		if @suiteB = 1
		begin		
			set @qatt_iter = 1;		
			while @qatt_iter <= @qatt_max
			begin
				select 
					@queriedAttributes = queriedAttributes,
					@queryType = queryType 
				from 
					TestMetadata.dbo.QueriedAttributes 
				where 
					iteration = @qatt_iter 
				and 
					numberOfAttributes = @numberOfAttributes;					
				
				select @query = '
				select
					count(*) as numberOfRecords
				'
				
				select @query = @query + '
				into
					#UDS_3NF_B'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ '
				from 
					[3nfAN_Anchor]			
				where	
					1 = 1		
				'

				set @qsub_iter = 1;

				while @qsub_iter <= @queriedAttributes
				begin
					select @query = @query 
					+ 'and ANATT_AnchorAttribute_' 
					+ cast(@qsub_iter as varchar(10)) 
					+ ' = 0 
					'
					
					set @qsub_iter = @qsub_iter + 1;
				end

				
				select @sql = @sql + '
				
				set @runs = ' + cast(@query_repeats as varchar(10)) + ';
				set @iter = 0;

				while @iter < @runs
				begin
					if object_id(''tempdb..#UDS_3NF_B'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ ''') is not null
					drop table #UDS_3NF_B'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ ';

					-- clear cache
					CHECKPOINT;
					DBCC DROPCLEANBUFFERS WITH NO_INFOMSGS;
					DBCC FREEPROCCACHE WITH NO_INFOMSGS;
					DBCC FREESESSIONCACHE WITH NO_INFOMSGS;
					DBCC FREESYSTEMCACHE (''All'') WITH NO_INFOMSGS;
					
					-- make disk cache dirty
					select top 0 
						*
					into 
						DirtyCache
					from
						TestData.dbo.DirtyCache;
					
					insert into DirtyCache	
					select
						*
					from 
						TestData.dbo.DirtyCache
						
					drop table DirtyCache;

					insert into TestMetadata.dbo.QueryTimes(
						suite,
						data,
						modelType,
						modeledAttributes,
						queryType,
						queryIteration,
						queriedAttributes,
						numberOfRows,
						startingTime
					)
					values(
						''B'',
						''UDS'',
						''3NF'',
						' + cast(@numberOfAttributes as varchar(10)) + ',
						''' + @queryType + ''',
						@iter + 1,
						' + cast(@queriedAttributes as varchar(10)) + ',
						' + cast(@numberOfRows as varchar(10)) + ',
						GETDATE()
					);
					
					' + @query + '

					update TestMetadata.dbo.QueryTimes 
					set
						endingTime = GETDATE()
					where
						endingTime is null
					and
						suite = ''B''
					and
						data = ''UDS''
					and
						modelType = ''3NF''
					and
						modeledAttributes = ' + cast(@numberOfAttributes as varchar(10)) + '
					and
						queryIteration = @iter + 1
					and
						queriedAttributes = ' + cast(@queriedAttributes as varchar(10)) + '
					and
						numberOfRows = ' + cast(@numberOfRows as varchar(10)) + '
						
					set @iter = @iter + 1;
				end			
				'
				
				set @qatt_iter = @qatt_iter + 1;
			end
		end

		select @sql = @sql + '
		DBCC SHRINKFILE (1, 1) -- data
		DBCC SHRINKFILE (2, 1) -- log
		
		declare @size bigint;
		select 
			@size = SUM(size) 
		from 
			Test3NF_UDS.sys.database_files 
		where 
			type_desc = ''ROWS'';
			
		insert into TestMetadata.dbo.DatabaseSizes (
			data,
			modelType,
			modeledAttributes,
			numberOfRows,
			size
		)
		values (
			''UDS'',
			''3NF'',
			' + cast(@numberOfAttributes as varchar(10)) + ',
			' + cast(@numberOfRows as varchar(10)) + ',
			@size
		);			

		use master;

		-- now we do not need the 3NF database any more
		IF  EXISTS (SELECT name FROM sys.databases WHERE name = N''Test3NF_UDS'')
		DROP DATABASE [Test3NF_UDS];

		-- create an empty db so that the script validation will not break
		create database [Test3NF_UDS];
		'

		insert into TestMetadata.dbo.Queries (
			data,
			modelType,
			modeledAttributes,
			numberOfRows,
			query	
		)
		values (
			'UDS',
			'3NF',
			@numberOfAttributes,
			@numberOfRows,
			@sql
		);
		
		set @rows_iter = @rows_iter + 1;
	end
	
	set @size_iter = @size_iter + 1;
end

/*
select * from TestMetadata.dbo.Queries

*/

-- time to run all tests: 3 hours

declare @current_id int;
declare @maximum_id int;

select 
	@current_id = MIN(id),
	@maximum_id = MAX(id)
from 
	TestMetadata.dbo.Queries
where
	data = 'UDS';

declare @nquery nvarchar(max);

/* Uncomment to override and continue a broken test suite */
--set @current_id = 15;
	
while @current_id <= @maximum_id
begin
	select 
		@nquery = query 
	from 
		TestMetadata.dbo.Queries
	where 
		id = @current_id;
		
	exec sp_executesql @nquery

	set @current_id = @current_id + 1;
end

