USE [master]
GO

-- heterogenous sparse historized

IF  EXISTS (SELECT name FROM sys.databases WHERE name = N'Test3NF_HSH')
DROP DATABASE [Test3NF_HSH];
create database [Test3NF_HSH];

IF  EXISTS (SELECT name FROM sys.databases WHERE name = N'TestAM_HSH')
DROP DATABASE [TestAM_HSH];
create database [TestAM_HSH];
GO

-- configuration parameters

declare @query_repeats int;
set @query_repeats = 2;

declare @suiteA bit;
set @suiteA = 1;

declare @suiteB bit;
set @suiteB = 1;

-- iteration counters

declare @rows_iter int;
declare @size_iter int;
declare @qatt_iter int;

set @rows_iter = 1;
set @size_iter = 1;
set @qatt_iter = 1;

declare @rows_max int;
declare @size_max int;
declare @qatt_max int;

select @rows_max = MAX(iteration) from TestMetadata.dbo.NumberOfRows;
select @size_max = MAX(iteration) from TestMetadata.dbo.NumberOfAttributes;
select @qatt_max = MAX(iteration) from TestMetadata.dbo.QueriedAttributes;


-- loop through the measure points

while @size_iter <= @size_max
begin
	declare @numberOfAttributes int;
	select 
		@numberOfAttributes = numberOfAttributes 
	from 
		TestMetadata.dbo.NumberOfAttributes 
	where 
		iteration = @size_iter;
	
	set @rows_iter = 1;
	while @rows_iter <= @rows_max
	begin	
		declare @numberOfRows int;
		select 
			@numberOfRows = numberOfRows 
		from 
			TestMetadata.dbo.NumberOfRows 
		where 
			iteration = @rows_iter;

		-- the query will be built into this variable
		declare @sql varchar(max);

		-- first build the anchor model query
		select @sql = '
		use master;
		
		IF  EXISTS (SELECT name FROM sys.databases WHERE name = N''TestAM_HSH'')
		DROP DATABASE [TestAM_HSH]

		CREATE DATABASE [TestAM_HSH] ON  PRIMARY ( 
			NAME = N''TestAM_HSH'', 
			FILENAME = N''D:\SQLData\TestAM_HSH.mdf'', 
			SIZE = 10GB, 
			MAXSIZE = UNLIMITED, 
			FILEGROWTH = 10GB 
		)
		LOG ON ( 
			NAME = N''TestAM_HSH_log'', 
			FILENAME = N''D:\SQLData\TestAM_HSH_log.ldf'', 
			SIZE = 5GB, 
			MAXSIZE = 2048GB, 
			FILEGROWTH = 5GB 
		)

		ALTER DATABASE [TestAM_HSH] SET COMPATIBILITY_LEVEL = 100
		ALTER DATABASE [TestAM_HSH] SET AUTO_CREATE_STATISTICS ON 
		ALTER DATABASE [TestAM_HSH] SET AUTO_UPDATE_STATISTICS ON 
		ALTER DATABASE [TestAM_HSH] SET RECOVERY SIMPLE 
		
		USE TestAM_HSH;

		create table KNO_Knot_1 (
			KNO_ID bit not null, 
			KNO_Value char(4) not null
		);
		
		insert into KNO_Knot_1
		select * from TestData.dbo.KNO_Knot_1;
		
		alter table KNO_Knot_1 add constraint pkKNO_Knot_1 primary key clustered (KNO_ID asc);
		alter table KNO_Knot_1 add constraint uqKNO_Knot_1 unique (KNO_Value);
		update statistics KNO_Knot_1;

		create table KNO_Knot_3 (
			KNO_ID tinyint not null,
			KNO_Value varchar(8) not null
		);
		
		insert into KNO_Knot_3
		select * from TestData.dbo.KNO_Knot_3;

		alter table KNO_Knot_3 add constraint pkKNO_Knot_3 primary key clustered (KNO_ID asc);
		alter table KNO_Knot_3 add constraint uqKNO_Knot_3 unique (KNO_Value);
		update statistics KNO_Knot_3;

		CREATE TABLE [AN_Anchor] ( 
			AN_ID int not null
		);		
		insert into AN_Anchor
		select top ' + cast(@numberOfRows as varchar(10)) + ' AN_ID from TestData.dbo.ANATT_AnchorAttribute_HDS_0;
		alter table AN_Anchor add constraint pkAN_Anchor primary key clustered (AN_ID asc);
		update statistics AN_Anchor;
		'

		declare @attr_iter int;
		set @attr_iter = 1;
		declare @dt_index int;
		
		while @attr_iter <= @numberOfAttributes
		begin
			select @dt_index = (@attr_iter - 1) % 6;
			select @sql = @sql + '
			CREATE TABLE [ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + '] ( 
				AN_ID int not null,
			'
			if @dt_index = 0
			begin
				select @sql = @sql + '
					ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' money not null,
					ANATT_ValidFrom_' + cast(@attr_iter as varchar(10)) + ' date not null
				);
				insert into ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + '
				select top ' + cast(@numberOfRows/2 as varchar(10)) + ' * from TestData.dbo.ANATT_AnchorAttribute_HDH_' + CAST(@dt_index as varchar(10)) + ' tablesample (50 percent) repeatable (42);
				alter table ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' add constraint pkANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' primary key clustered (AN_ID asc, ANATT_ValidFrom_' + cast(@attr_iter as varchar(10)) + ' desc);
				alter table ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' add constraint fkANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' foreign key (AN_ID) references AN_Anchor (AN_ID);
				update statistics ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ';
				'
			end
			if @dt_index = 1
			begin
				select @sql = @sql + '
					KNO_ID bit not null
				);
				alter table ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' add constraint fkANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + '_KNO foreign key (KNO_ID) references KNO_Knot_1 (KNO_ID);
				insert into ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + '
				select top ' + cast(@numberOfRows/2 as varchar(10)) + ' * from TestData.dbo.ANATT_AnchorAttribute_HDS_' + CAST(@dt_index as varchar(10)) + ' tablesample (50 percent) repeatable (42);
				alter table ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' add constraint pkANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' primary key clustered (AN_ID asc);
				alter table ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' add constraint fkANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' foreign key (AN_ID) references AN_Anchor (AN_ID);
				update statistics ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ';
				'
			end
			if @dt_index = 2
			begin
				select @sql = @sql + '
					ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' varchar(10) not null
				);
				insert into ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + '
				select top ' + cast(@numberOfRows/2 as varchar(10)) + ' * from TestData.dbo.ANATT_AnchorAttribute_HDS_' + CAST(@dt_index as varchar(10)) + ' tablesample (50 percent) repeatable (42);
				alter table ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' add constraint pkANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' primary key clustered (AN_ID asc);
				alter table ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' add constraint fkANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' foreign key (AN_ID) references AN_Anchor (AN_ID);
				update statistics ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ';
				'
			end
			if @dt_index = 3
			begin
				select @sql = @sql + '
					KNO_ID tinyint not null
				);
				alter table ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' add constraint fkANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + '_KNO foreign key (KNO_ID) references KNO_Knot_3 (KNO_ID);
				insert into ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + '
				select top ' + cast(@numberOfRows/2 as varchar(10)) + ' * from TestData.dbo.ANATT_AnchorAttribute_HDS_' + CAST(@dt_index as varchar(10)) + ' tablesample (50 percent) repeatable (42);
				alter table ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' add constraint pkANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' primary key clustered (AN_ID asc);
				alter table ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' add constraint fkANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' foreign key (AN_ID) references AN_Anchor (AN_ID);
				update statistics ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ';
				'
			end
			if @dt_index = 4
			begin
				select @sql = @sql + '
					ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' datetime not null
				);
				insert into ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + '
				select top ' + cast(@numberOfRows/2 as varchar(10)) + ' * from TestData.dbo.ANATT_AnchorAttribute_HDS_' + CAST(@dt_index as varchar(10)) + ' tablesample (50 percent) repeatable (42);
				alter table ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' add constraint pkANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' primary key clustered (AN_ID asc);
				alter table ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' add constraint fkANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' foreign key (AN_ID) references AN_Anchor (AN_ID);
				update statistics ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ';
				'
			end
			if @dt_index = 5
			begin
				select @sql = @sql + '
					ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' char(32) not null
				);
				insert into ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + '
				select top ' + cast(@numberOfRows/2 as varchar(10)) + ' * from TestData.dbo.ANATT_AnchorAttribute_HDS_' + CAST(@dt_index as varchar(10)) + ' tablesample (50 percent) repeatable (42);
				alter table ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' add constraint pkANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' primary key clustered (AN_ID asc);
				alter table ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' add constraint fkANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' foreign key (AN_ID) references AN_Anchor (AN_ID);
				update statistics ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ';
				'
			end
			
			set @attr_iter = @attr_iter + 1;
		end

		select @sql = @sql + '
		exec TestAM_HSH.dbo.sp_executesql N''
		create view lAN_Anchor as 
		SELECT 
			[AN].AN_ID				
		'

		set @attr_iter = 1;

		while @attr_iter <= @numberOfAttributes
		begin
			select @dt_index = (@attr_iter - 1) % 6;
			if @dt_index in (0, 2, 4, 5)
			begin
				select @sql = 
				@sql + ',[' + cast(@attr_iter as varchar(10)) + '].ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + '
				'
			end
			if @dt_index in (1, 3)
			begin
				select @sql = 
				@sql + ',[k' + cast(@attr_iter as varchar(10)) + '].KNO_Value as KNO_Value_' + cast(@attr_iter as varchar(10)) + '
				'
			end
			
			set @attr_iter = @attr_iter + 1;
		end

		select @sql = @sql + '
		from 
			AN_Anchor [AN]
		'

		set @attr_iter = 1;

		while @attr_iter <= @numberOfAttributes
		begin
			select @dt_index = (@attr_iter - 1) % 6;
			select @sql = 
			@sql + 'left join ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' [' + cast(@attr_iter as varchar(10)) + ']
			on [' + cast(@attr_iter as varchar(10)) + '].AN_ID = [AN].AN_ID
			'
			if @dt_index = 0
			begin
				select @sql = 
				@sql + 'and [' + cast(@attr_iter as varchar(10)) + '].ANATT_ValidFrom_' + cast(@attr_iter as varchar(10)) + '
				= (
					select
						max(sub.ANATT_ValidFrom_' + cast(@attr_iter as varchar(10)) + ')
					from
						ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' sub
					where
						sub.AN_ID = [AN].AN_ID
				)
				'
			end
			if @dt_index in (1, 3)
			begin
				select @sql = 
				@sql + 'left join KNO_Knot_' + cast(@dt_index as char(1)) + ' [k' + cast(@attr_iter as varchar(10)) + ']
				on [k' + cast(@attr_iter as varchar(10)) + '].KNO_ID = [' + cast(@attr_iter as varchar(10)) + '].KNO_ID
				'
			end			
			
			set @attr_iter = @attr_iter + 1;
		end
		
		select @sql = @sql + '''
		'

		select @sql = @sql + '
		exec TestAM_HSH.dbo.sp_executesql N''
		create function pAN_Anchor (@timepoint datetime)
		RETURNS TABLE RETURN 
		SELECT 
			[AN].AN_ID				
		'

		set @attr_iter = 1;

		while @attr_iter <= @numberOfAttributes
		begin
			select @dt_index = (@attr_iter - 1) % 6;
			if @dt_index in (0, 2, 4, 5)
			begin
				select @sql = 
				@sql + ',[' + cast(@attr_iter as varchar(10)) + '].ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + '
				'
			end
			if @dt_index = 0
			begin
				select @sql = 
				@sql + ',[' + cast(@attr_iter as varchar(10)) + '].ANATT_ValidFrom_' + cast(@attr_iter as varchar(10)) + '
				'
			end
			if @dt_index in (1, 3)
			begin
				select @sql = 
				@sql + ',[k' + cast(@attr_iter as varchar(10)) + '].KNO_Value as KNO_Value_' + cast(@attr_iter as varchar(10)) + '
				'
			end
			
			set @attr_iter = @attr_iter + 1;
		end

		select @sql = @sql + '
		from 
			AN_Anchor [AN]
		'

		set @attr_iter = 1;

		while @attr_iter <= @numberOfAttributes
		begin
			select @dt_index = (@attr_iter - 1) % 6;
			select @sql = 
			@sql + 'left join ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' [' + cast(@attr_iter as varchar(10)) + ']
			on [' + cast(@attr_iter as varchar(10)) + '].AN_ID = [AN].AN_ID
			'
			if @dt_index = 0
			begin
				select @sql = 
				@sql + 'and [' + cast(@attr_iter as varchar(10)) + '].ANATT_ValidFrom_' + cast(@attr_iter as varchar(10)) + '
				= (
					select
						max(sub.ANATT_ValidFrom_' + cast(@attr_iter as varchar(10)) + ')
					from
						ANATT_AnchorAttribute_' + cast(@attr_iter as varchar(10)) + ' sub
					where
						sub.AN_ID = [AN].AN_ID
					and
						sub.ANATT_ValidFrom_' + cast(@attr_iter as varchar(10)) + ' <= @timepoint
				)
				'
			end
			if @dt_index in (1, 3)
			begin
				select @sql = 
				@sql + 'left join KNO_Knot_' + cast(@dt_index as char(1)) + ' [k' + cast(@attr_iter as varchar(10)) + ']
				on [k' + cast(@attr_iter as varchar(10)) + '].KNO_ID = [' + cast(@attr_iter as varchar(10)) + '].KNO_ID
				'
			end			
			
			set @attr_iter = @attr_iter + 1;
		end
		
		select @sql = @sql + '''
		
		declare @iter int;
		declare @runs int;
		declare @identity int;		
		'

		declare @query varchar(max);
		declare @queriedAttributes int;
		declare @queryType varchar(255);
		declare @qsub_iter int;

		-- without where conditions
		if @suiteA = 1
		begin
			set @qatt_iter = 1;		
			while @qatt_iter <= @qatt_max
			begin
				select 
					@queriedAttributes = queriedAttributes,
					@queryType = queryType
				from 
					TestMetadata.dbo.QueriedAttributes 
				where 
					iteration = @qatt_iter 
				and 
					numberOfAttributes = @numberOfAttributes;					
				
				select @query = '
				select
					0 as dummy
				'
				
				set @qsub_iter = 1;

				while @qsub_iter <= @queriedAttributes
				begin
					select @dt_index = (@qsub_iter - 1) % 6;
					if @dt_index = 0
					begin
						select @query = @query 
						+ ', count(case when ANATT_AnchorAttribute_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ' > 0 then 1 else null end) as value_' 
						+ cast(@qsub_iter as varchar(10)) + '
						'
					end
					if @dt_index = 1
					begin
						select @query = @query 
						+ ', count(case when KNO_Value_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ' = ''Even'' then 1 else null end) as value_' 
						+ cast(@qsub_iter as varchar(10)) + '
						'
					end
					if @dt_index = 2
					begin
						select @query = @query 
						+ ', count(case when RIGHT(ANATT_AnchorAttribute_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ', 1) = ''0'' then 1 else null end) as value_' 
						+ cast(@qsub_iter as varchar(10)) + '
						'
					end
					if @dt_index = 3
					begin
						select @query = @query 
						+ ', count(case when KNO_Value_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ' = ''Alpha'' then 1 else null end) as value_' 
						+ cast(@qsub_iter as varchar(10)) + '
						'
					end
					if @dt_index = 4
					begin
						select @query = @query 
						+ ', count(case when ANATT_AnchorAttribute_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ' > ''2010-01-01'' then 1 else null end) as value_' 
						+ cast(@qsub_iter as varchar(10)) + '
						'
					end
					if @dt_index = 5
					begin
						select @query = @query 
						+ ', count(case when LEFT(ANATT_AnchorAttribute_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ', 1) = ''A'' then 1 else null end) as value_' 
						+ cast(@qsub_iter as varchar(10)) + '
						'
					end
					
					set @qsub_iter = @qsub_iter + 1;
				end

				select @query = @query + '
				into
					#HSH_AM_A'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ '
				from 
					lAN_Anchor;			
				'
				
				select @sql = @sql + '
				
				set @runs = ' + cast(@query_repeats as varchar(10)) + '
				set @iter = 0;

				while @iter < @runs
				begin
					if object_id(''tempdb..#HSH_AM_A'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ ''') is not null
					drop table #HSH_AM_A'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ ';
				
					-- clear cache
					CHECKPOINT;
					DBCC DROPCLEANBUFFERS WITH NO_INFOMSGS;
					DBCC FREEPROCCACHE WITH NO_INFOMSGS;
					DBCC FREESESSIONCACHE WITH NO_INFOMSGS;
					DBCC FREESYSTEMCACHE (''All'') WITH NO_INFOMSGS;

					-- make disk cache dirty
					select top 0 
						*
					into 
						DirtyCache
					from
						TestData.dbo.DirtyCache;
					
					insert into DirtyCache	
					select
						*
					from 
						TestData.dbo.DirtyCache
						
					drop table DirtyCache;

					insert into TestMetadata.dbo.QueryTimes(
						suite,
						data,
						modelType,
						modeledAttributes,
						queryType,
						queryIteration,
						queriedAttributes,
						numberOfRows,
						startingTime
					)
					values(
						''A'',
						''HSH'',
						''AM'',
						' + cast(@numberOfAttributes as varchar(10)) + ',
						''' + @queryType + ''',
						@iter + 1,
						' + cast(@queriedAttributes as varchar(10)) + ',
						' + cast(@numberOfRows as varchar(10)) + ',
						GETDATE()
					);
					
					' + @query + '

					update TestMetadata.dbo.QueryTimes 
					set
						endingTime = GETDATE()
					where
						endingTime is null
					and
						suite = ''A''
					and
						data = ''HSH''
					and
						modelType = ''AM''
					and
						modeledAttributes = ' + cast(@numberOfAttributes as varchar(10)) + '
					and
						queryIteration = @iter + 1
					and
						queriedAttributes = ' + cast(@queriedAttributes as varchar(10)) + '
					and
						numberOfRows = ' + cast(@numberOfRows as varchar(10)) + '
						
					set @iter = @iter + 1;
				end			
				'
				
				set @qatt_iter = @qatt_iter + 1;
			end
		end

		-- with where conditions
		if @suiteB = 1
		begin
			set @qatt_iter = 1;		
			while @qatt_iter <= @qatt_max
			begin
				select 
					@queriedAttributes = queriedAttributes,
					@queryType = queryType
				from 
					TestMetadata.dbo.QueriedAttributes 
				where 
					iteration = @qatt_iter 
				and 
					numberOfAttributes = @numberOfAttributes;					
				
				select @query = '
				select
					count(*) as numberOfRecords
				'				

				select @query = @query + '
				into
					#HSH_AM_B'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ '
				from 
					lAN_Anchor
				where 
					1 = 1		
				'

				set @qsub_iter = 1;

				while @qsub_iter <= @queriedAttributes
				begin
					select @dt_index = (@qsub_iter - 1) % 6;
					if @dt_index = 0
					begin
						select @query = @query 
						+ 'and ANATT_AnchorAttribute_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ' > 0 
						'
					end
					if @dt_index = 1
					begin
						select @query = @query 
						+ 'and KNO_Value_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ' = ''Even''
						'					
					end
					if @dt_index = 2
					begin
						select @query = @query 
						+ 'and RIGHT(ANATT_AnchorAttribute_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ', 1) = ''0'' 
						' 
					end
					if @dt_index = 3
					begin
						select @query = @query 
						+ 'and KNO_Value_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ' = ''Alpha'' 
						'
					end
					if @dt_index = 4
					begin
						select @query = @query 
						+ 'and ANATT_AnchorAttribute_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ' > ''2010-01-01''
						' 
					end
					if @dt_index = 5
					begin
						select @query = @query 
						+ 'and LEFT(ANATT_AnchorAttribute_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ', 1) = ''A'' 
						' 
					end
					
					set @qsub_iter = @qsub_iter + 1;
				end

				
				select @sql = @sql + '
				
				set @runs = ' + cast(@query_repeats as varchar(10)) + '
				set @iter = 0;

				while @iter < @runs
				begin
					if object_id(''tempdb..#HSH_AM_B'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ ''') is not null
					drop table #HSH_AM_B'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ ';
				
					-- clear cache
					CHECKPOINT;
					DBCC DROPCLEANBUFFERS WITH NO_INFOMSGS;
					DBCC FREEPROCCACHE WITH NO_INFOMSGS;
					DBCC FREESESSIONCACHE WITH NO_INFOMSGS;
					DBCC FREESYSTEMCACHE (''All'') WITH NO_INFOMSGS;

					-- make disk cache dirty
					select top 0 
						*
					into 
						DirtyCache
					from
						TestData.dbo.DirtyCache;
					
					insert into DirtyCache	
					select
						*
					from 
						TestData.dbo.DirtyCache
						
					drop table DirtyCache;

					insert into TestMetadata.dbo.QueryTimes(
						suite,
						data,
						modelType,
						modeledAttributes,
						queryType,
						queryIteration,
						queriedAttributes,
						numberOfRows,
						startingTime
					)
					values(
						''B'',
						''HSH'',
						''AM'',
						' + cast(@numberOfAttributes as varchar(10)) + ',
						''' + @queryType + ''',
						@iter + 1,
						' + cast(@queriedAttributes as varchar(10)) + ',
						' + cast(@numberOfRows as varchar(10)) + ',
						GETDATE()
					);
					
					' + @query + '

					update TestMetadata.dbo.QueryTimes 
					set
						endingTime = GETDATE()
					where
						endingTime is null
					and
						suite = ''B''
					and
						data = ''HSH''
					and
						modelType = ''AM''
					and
						modeledAttributes = ' + cast(@numberOfAttributes as varchar(10)) + '
					and
						queryIteration = @iter + 1
					and
						queriedAttributes = ' + cast(@queriedAttributes as varchar(10)) + '
					and
						numberOfRows = ' + cast(@numberOfRows as varchar(10)) + '
						
					set @iter = @iter + 1;
				end			
				'
				
				set @qatt_iter = @qatt_iter + 1;
			end
		end
		
		select @sql = @sql + '
		DBCC SHRINKFILE (1, 1) -- data
		DBCC SHRINKFILE (2, 1) -- log
		
		declare @size bigint;
		select 
			@size = SUM(size) 
		from 
			TestAM_HSH.sys.database_files 
		where 
			type_desc = ''ROWS'';

		insert into TestMetadata.dbo.DatabaseSizes (
			data,
			modelType,
			modeledAttributes,
			numberOfRows,
			size
		)
		values (
			''HSH'',
			''AM'',
			' + cast(@numberOfAttributes as varchar(10)) + ',
			' + cast(@numberOfRows as varchar(10)) + ',
			@size
		);			
		'
		
		insert into TestMetadata.dbo.Queries (
			data,
			modelType,
			modeledAttributes,
			numberOfRows,
			query	
		)
		values (
			'HSH',
			'AM',
			@numberOfAttributes,
			@numberOfRows,
			@sql
		);
		
		-- then build the 3NF query
		set @sql = '
		use master;
		
		IF  EXISTS (SELECT name FROM sys.databases WHERE name = N''Test3NF_HSH'')
		DROP DATABASE [Test3NF_HSH]

		CREATE DATABASE [Test3NF_HSH] ON  PRIMARY ( 
			NAME = N''Test3NF_HSH'', 
			FILENAME = N''D:\SQLData\Test3NF_HSH.mdf'', 
			SIZE = 10GB, 
			MAXSIZE = UNLIMITED, 
			FILEGROWTH = 10GB 
		)
		LOG ON ( 
			NAME = N''Test3NF_HSH_log'', 
			FILENAME = N''D:\SQLData\Test3NF_HSH_log.ldf'', 
			SIZE = 5GB, 
			MAXSIZE = 2048GB, 
			FILEGROWTH = 5GB 
		)

		ALTER DATABASE [Test3NF_HSH] SET COMPATIBILITY_LEVEL = 100
		ALTER DATABASE [Test3NF_HSH] SET AUTO_CREATE_STATISTICS ON 
		ALTER DATABASE [Test3NF_HSH] SET AUTO_UPDATE_STATISTICS ON 
		ALTER DATABASE [Test3NF_HSH] SET RECOVERY SIMPLE 
		
		use Test3NF_HSH;
		
		select top 0
			*
		into
			[3nfAN_Anchor]
		from
			TestAM_HSH.dbo.pAN_Anchor(''1972-08-20'');
			
		declare @insert_iter int;
		declare @insert_max int;		
		declare @from_id int;
		declare @to_id int;
				
		set @insert_iter = 1;
		set @insert_max = 100;

		while @insert_iter <= @insert_max 
		begin
			select @from_id = (@insert_iter - 1) * (' + cast(@numberOfRows as varchar(10)) + ' / @insert_max) + 1;
			select @to_id = @insert_iter * (' + cast(@numberOfRows as varchar(10)) + ' / @insert_max);

			insert into [3nfAN_Anchor]
			select 
				*
			from
				TestAM_HSH.dbo.pAN_Anchor(''1972-08-20'')
			where
				AN_ID between @from_id and @to_id;

			insert into [3nfAN_Anchor]
			select 
				*
			from
				TestAM_HSH.dbo.pAN_Anchor(''2014-06-19'')
			where
				AN_ID between @from_id and @to_id;
				
			set @insert_iter = @insert_iter + 1;
		end

		-- now we do not need the AM database any longer
		IF  EXISTS (SELECT name FROM sys.databases WHERE name = N''TestAM_HSH'')
		DROP DATABASE [TestAM_HSH];
		
		-- create an empty db so that the script validation will not break
		create database [TestAM_HSH];

		create clustered index cl3nfAN_Anchor on [3nfAN_Anchor] (AN_ID asc);
		update statistics [3nfAN_Anchor];
						
		declare @iter int;
		declare @runs int;
		declare @identity int;		
		'
		
		set @attr_iter = 1;

		while @attr_iter <= @numberOfAttributes
		begin
			select @dt_index = (@attr_iter - 1) % 6;
			if @dt_index = 0
			begin
				select @sql = @sql + '
				create index v' + cast(@attr_iter as varchar(10)) + '3nfAN_Anchor on [3nfAN_Anchor] 
				(ANATT_ValidFrom_' + cast(@attr_iter as varchar(10)) + ' desc);
				update statistics [3nfAN_Anchor];
				'
			end
			
			set @attr_iter = @attr_iter + 1;
		end


		set @query ='';
		
		-- without where conditions
		if @suiteA = 1
		begin
			set @qatt_iter = 1;		
			while @qatt_iter <= @qatt_max
			begin
				select 
					@queriedAttributes = queriedAttributes,
					@queryType = queryType 
				from 
					TestMetadata.dbo.QueriedAttributes 
				where 
					iteration = @qatt_iter 
				and 
					numberOfAttributes = @numberOfAttributes;					
				
				select @query = '
				select
					0 as dummy
				'
				
				set @qsub_iter = 1;

				while @qsub_iter <= @queriedAttributes
				begin
					select @dt_index = (@qsub_iter - 1) % 6;
					if @dt_index = 0
					begin
						select @query = @query 
						+ ', count(case when ANATT_AnchorAttribute_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ' > 0 then 1 else null end) as value_' 
						+ cast(@qsub_iter as varchar(10)) + '
						'
					end
					if @dt_index = 1
					begin
						select @query = @query 
						+ ', count(case when KNO_Value_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ' = ''Even'' then 1 else null end) as value_' 
						+ cast(@qsub_iter as varchar(10)) + '
						'
					end
					if @dt_index = 2
					begin
						select @query = @query 
						+ ', count(case when RIGHT(ANATT_AnchorAttribute_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ', 1) = ''0'' then 1 else null end) as value_' 
						+ cast(@qsub_iter as varchar(10)) + '
						'
					end
					if @dt_index = 3
					begin
						select @query = @query 
						+ ', count(case when KNO_Value_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ' = ''Alpha'' then 1 else null end) as value_' 
						+ cast(@qsub_iter as varchar(10)) + '
						'
					end
					if @dt_index = 4
					begin
						select @query = @query 
						+ ', count(case when ANATT_AnchorAttribute_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ' > ''2010-01-01'' then 1 else null end) as value_' 
						+ cast(@qsub_iter as varchar(10)) + '
						'
					end
					if @dt_index = 5
					begin
						select @query = @query 
						+ ', count(case when LEFT(ANATT_AnchorAttribute_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ', 1) = ''A'' then 1 else null end) as value_' 
						+ cast(@qsub_iter as varchar(10)) + '
						'
					end
					
					set @qsub_iter = @qsub_iter + 1;
				end

				select @query = @query + '
				into
					#HSH_3NF_A'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ '
				from 
					[3nfAN_Anchor] AN
				where
					1 = 1			
				'

				set @qsub_iter = 1;

				while @qsub_iter <= @queriedAttributes
				begin
					select @dt_index = (@qsub_iter - 1) % 6;
					if @dt_index = 0
					begin
						select @query = @query 
						+ 'and ANATT_ValidFrom_' + cast(@qsub_iter as varchar(10)) 
						+ ' = (
							select
								max(sub.ANATT_ValidFrom_' + cast(@qsub_iter as varchar(10)) + ')
							from
								[3nfAN_Anchor] sub
							where
								sub.AN_ID = AN.AN_ID
							)
						'
					end
					
					set @qsub_iter = @qsub_iter + 1;
				end
				
				select @sql = @sql + '
				
				set @runs = ' + cast(@query_repeats as varchar(10)) + ';
				set @iter = 0;

				while @iter < @runs
				begin
					if object_id(''tempdb..#HSH_3NF_A'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ ''') is not null
					drop table #HSH_3NF_A'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ ';

					-- clear cache
					CHECKPOINT;
					DBCC DROPCLEANBUFFERS WITH NO_INFOMSGS;
					DBCC FREEPROCCACHE WITH NO_INFOMSGS;
					DBCC FREESESSIONCACHE WITH NO_INFOMSGS;
					DBCC FREESYSTEMCACHE (''All'') WITH NO_INFOMSGS;

					-- make disk cache dirty
					select top 0 
						*
					into 
						DirtyCache
					from
						TestData.dbo.DirtyCache;
					
					insert into DirtyCache	
					select
						*
					from 
						TestData.dbo.DirtyCache
						
					drop table DirtyCache;

					insert into TestMetadata.dbo.QueryTimes(
						suite,
						data,
						modelType,
						modeledAttributes,
						queryType,
						queryIteration,
						queriedAttributes,
						numberOfRows,
						startingTime
					)
					values(
						''A'',
						''HSH'',
						''3NF'',
						' + cast(@numberOfAttributes as varchar(10)) + ',
						''' + @queryType + ''',
						@iter + 1,
						' + cast(@queriedAttributes as varchar(10)) + ',
						' + cast(@numberOfRows as varchar(10)) + ',
						GETDATE()
					);
					
					' + @query + '

					update TestMetadata.dbo.QueryTimes 
					set
						endingTime = GETDATE()
					where
						endingTime is null
					and
						suite = ''A''
					and
						data = ''HSH''
					and
						modelType = ''3NF''
					and
						modeledAttributes = ' + cast(@numberOfAttributes as varchar(10)) + '
					and
						queryIteration = @iter + 1
					and
						queriedAttributes = ' + cast(@queriedAttributes as varchar(10)) + '
					and
						numberOfRows = ' + cast(@numberOfRows as varchar(10)) + '
						
					set @iter = @iter + 1;
				end			
				'
				
				set @qatt_iter = @qatt_iter + 1;
			end
		end

		-- with where conditions
		if @suiteB = 1
		begin
			set @qatt_iter = 1;		
			while @qatt_iter <= @qatt_max
			begin
				select 
					@queriedAttributes = queriedAttributes,
					@queryType = queryType 
				from 
					TestMetadata.dbo.QueriedAttributes 
				where 
					iteration = @qatt_iter 
				and 
					numberOfAttributes = @numberOfAttributes;					
				
				select @query = '
				select
					count(*) as numberOfRecords
				'				

				select @query = @query + '
				into
					#HSH_3NF_B'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ '
				from 
					[3nfAN_Anchor] AN
				where
					1 = 1		
				'

				set @qsub_iter = 1;

				while @qsub_iter <= @queriedAttributes
				begin
					select @dt_index = (@qsub_iter - 1) % 6;
					if @dt_index = 0
					begin
						select @query = @query 
						+ 'and ANATT_AnchorAttribute_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ' > 0 
						 and ANATT_ValidFrom_' + cast(@qsub_iter as varchar(10)) + ' = (
							select
								max(sub.ANATT_ValidFrom_' + cast(@qsub_iter as varchar(10)) + ')
							from
								[3nfAN_Anchor] sub
							where
								sub.AN_ID = AN.AN_ID
							)
						'
					end
					if @dt_index = 1
					begin
						select @query = @query 
						+ 'and KNO_Value_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ' = ''Even''
						'					
					end
					if @dt_index = 2
					begin
						select @query = @query 
						+ 'and RIGHT(ANATT_AnchorAttribute_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ', 1) = ''0'' 
						' 
					end
					if @dt_index = 3
					begin
						select @query = @query 
						+ 'and KNO_Value_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ' = ''Alpha'' 
						'
					end
					if @dt_index = 4
					begin
						select @query = @query 
						+ 'and ANATT_AnchorAttribute_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ' > ''2010-01-01''
						' 
					end
					if @dt_index = 5
					begin
						select @query = @query 
						+ 'and LEFT(ANATT_AnchorAttribute_' 
						+ cast(@qsub_iter as varchar(10)) 
						+ ', 1) = ''A'' 
						' 
					end
					
					set @qsub_iter = @qsub_iter + 1;
				end
				
				select @sql = @sql + '
				
				set @runs = ' + cast(@query_repeats as varchar(10)) + ';
				set @iter = 0;

				while @iter < @runs
				begin
					if object_id(''tempdb..#HSH_3NF_B'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ ''') is not null
					drop table #HSH_3NF_B'  
					+ '_' + cast(@numberOfAttributes as varchar(10)) 
					+ '_' + cast(@numberOfRows as varchar(10))
					+ '_' + cast(@queriedAttributes as varchar(10))
					+ '_' + cast(@qatt_iter as varchar(10))
					+ ';

					-- clear cache
					CHECKPOINT;
					DBCC DROPCLEANBUFFERS WITH NO_INFOMSGS;
					DBCC FREEPROCCACHE WITH NO_INFOMSGS;
					DBCC FREESESSIONCACHE WITH NO_INFOMSGS;
					DBCC FREESYSTEMCACHE (''All'') WITH NO_INFOMSGS;

					-- make disk cache dirty
					select top 0 
						*
					into 
						DirtyCache
					from
						TestData.dbo.DirtyCache;
					
					insert into DirtyCache	
					select
						*
					from 
						TestData.dbo.DirtyCache
						
					drop table DirtyCache;

					insert into TestMetadata.dbo.QueryTimes(
						suite,
						data,
						modelType,
						modeledAttributes,
						queryType,
						queryIteration,
						queriedAttributes,
						numberOfRows,
						startingTime
					)
					values(
						''B'',
						''HSH'',
						''3NF'',
						' + cast(@numberOfAttributes as varchar(10)) + ',
						''' + @queryType + ''',
						@iter + 1,
						' + cast(@queriedAttributes as varchar(10)) + ',
						' + cast(@numberOfRows as varchar(10)) + ',
						GETDATE()
					);
					
					' + @query + '

					update TestMetadata.dbo.QueryTimes 
					set
						endingTime = GETDATE()
					where
						endingTime is null
					and
						suite = ''B''
					and
						data = ''HSH''
					and
						modelType = ''3NF''
					and
						modeledAttributes = ' + cast(@numberOfAttributes as varchar(10)) + '
					and
						queryIteration = @iter + 1
					and
						queriedAttributes = ' + cast(@queriedAttributes as varchar(10)) + '
					and
						numberOfRows = ' + cast(@numberOfRows as varchar(10)) + '
						
					set @iter = @iter + 1;
				end			
				'
				
				set @qatt_iter = @qatt_iter + 1;
			end
		end
		
		select @sql = @sql + '
		DBCC SHRINKFILE (1, 1) -- data
		DBCC SHRINKFILE (2, 1) -- log
		
		declare @size bigint;
		select 
			@size = SUM(size) 
		from 
			Test3NF_HSH.sys.database_files 
		where 
			type_desc = ''ROWS'';
			
		insert into TestMetadata.dbo.DatabaseSizes (
			data,
			modelType,
			modeledAttributes,
			numberOfRows,
			size
		)
		values (
			''HSH'',
			''3NF'',
			' + cast(@numberOfAttributes as varchar(10)) + ',
			' + cast(@numberOfRows as varchar(10)) + ',
			@size
		);			

		use master;

		-- now we do not need the 3NF database any more
		IF  EXISTS (SELECT name FROM sys.databases WHERE name = N''Test3NF_HSH'')
		DROP DATABASE [Test3NF_HSH];

		-- create an empty db so that the script validation will not break
		create database [Test3NF_HSH];
		'

		insert into TestMetadata.dbo.Queries (
			data,
			modelType,
			modeledAttributes,
			numberOfRows,
			query	
		)
		values (
			'HSH',
			'3NF',
			@numberOfAttributes,
			@numberOfRows,
			@sql
		);
		
		set @rows_iter = @rows_iter + 1;
	end
	
	set @size_iter = @size_iter + 1;
end


-- time to run all tests: 4+ hours

declare @current_id int;
declare @maximum_id int;

select 
	@current_id = MIN(id),
	@maximum_id = MAX(id)
from 
	TestMetadata.dbo.Queries
where
	data = 'HSH';

declare @nquery nvarchar(max);

/* Uncomment to override and continue a broken test suite */
--set @current_id = 407;
	
while @current_id <= @maximum_id
begin
	select 
		@nquery = query 
	from 
		TestMetadata.dbo.Queries
	where 
		id = @current_id;
		
	exec sp_executesql @nquery

	set @current_id = @current_id + 1;
end

